==> frontend_routing-1.0.0.tar.gz: frontend_routing/src/Form/SettingsForm.php <==
<?php

declare(strict_types=1);

namespace Drupal\frontend_routing\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Language\LanguageInterface;
use Drupal\Core\State\StateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure Frontend routing settings for this site.
 */
final class SettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function __construct(
    protected ConfigFactoryInterface $config_factory,
    protected StateInterface $state,
    protected EntityTypeManagerInterface $entityTypeManager
  ) {
    parent::__construct($config_factory);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('state'),
      $container->get('entity_type.manager'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'frontend_routing_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['frontend_routing.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {

    $config = $this->config('frontend_routing.settings');
    $nodes = $this->state->get('frontend_routing.settings') ?? [];

    $keys = $config->get('keys') ?? [];

    $form['existing'] = [
      '#type' => 'details',
      '#title' => $this->t('Existing mappings'),
      '#open' => TRUE,
    ];

    $headers = [
      $this->t('Key'),
      $this->t('Node'),
      $this->t('Language'),
      $this->t('Alias'),
    ];

    $form['existing']['table'] = [
      '#type' => 'table',
      '#header' => $headers,
      '#tree' => TRUE,
      '#empty' => $this->t('No mappings found.'),
    ];

    foreach ($keys as $key => $value) {
      $aliases = $value['aliases'] ?? [];

      $form['existing']['table'][$key]['key'] = [
        '#title' => $this->t('Key'),
        '#type' => 'textfield',
        '#default_value' => $key,
        '#disabled' => TRUE,
      ];

      $form['existing']['table'][$key]['entity_id'] = [
        '#title' => $this->t('Node'),
        '#type' => 'entity_autocomplete',
        '#target_type' => 'node',
        '#required' => TRUE,
        '#default_value' => !empty($nodes[$key]) ? $this->entityTypeManager->getStorage('node')->load($nodes[$key]) : NULL,
      ];

      $form['existing']['table'][$key]['alias'] = [
        '#title' => $this->t('Aliases'),
        '#type' => 'item',
        // '#default_value' => $alias,
        '#markup' => implode(',', $aliases),
      ];

      $form['existing']['table'][$key]['delete'] = [
        '#value' => $this->t('Delete'),
        '#type' => 'submit',
        '#limit_validation_errors' => [],
        '#submit' => [
          [$this, 'deleteEntry'],
        ],
      ];

    }

    $form['details'] = [
      '#type' => 'details',
      '#title' => $this->t('Add new mapping'),
      '#open' => TRUE,
    ];

    $form['details']['key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Key'),
      '#description' => 'Unique key to identify the mapping.',
      '#maxlength' => 255,
    ];

    $form['details']['entity_id'] = [
      '#type' => 'entity_autocomplete',
      '#title' => $this->t('Node'),
      '#target_type' => 'node',
      '#maxlength' => 255,
    ];

    $form['details']['language'] = [
      '#type' => 'language_select',
      '#title' => $this->t('Language'),
      '#languages' => LanguageInterface::STATE_CONFIGURABLE,
    ];

    $form['details']['alias'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Alias'),
      '#description' => 'Starts with a /.',
      '#maxlength' => 255,
    ];
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {

    if ($form_state->getValue('key')) {
      if (empty($form_state->getValue('entity_id'))) {
        $form_state->setErrorByName('entity_id', $this->t('Node is required.'));
      }
      if (empty($form_state->getValue('language'))) {
        $form_state->setErrorByName('language', $this->t('Language is required.'));
      }
      if (empty($form_state->getValue('alias'))) {
        $form_state->setErrorByName('alias', $this->t('Alias is required.'));
      }
    }

    parent::validateForm($form, $form_state);
  }

  /**
   * Deletes an entry from the frontend_routing settings.
   *
   * @param array &$form
   *   The form array passed by reference.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object containing the user input and other data.
   *
   * @throws \Drupal\Core\Config\ConfigException
   *   In case of configuration errors.
   */
  public function deleteEntry(array &$form, FormStateInterface $form_state): void {
    $table = $form_state->getValue('table');
    $keys = $this->config('frontend_routing.settings')->get('keys') ?? [];

    // Remove the selected keys.
    foreach ($table as $key => $value) {
      unset($keys[$key]);
    }
    $this->config('frontend_routing.settings')
      ->set('keys', $keys)
      ->save();
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {

    $nodes = [];
    // Save data from the table
    $keys = $this->config('frontend_routing.settings')->get('keys') ?? [];

    // Add a new item.
    $values = $form_state->getValues();
    if (!empty($values['key'])) {
      $key = !empty($keys[$values['key']]) ? $keys[$values['key']] : [];
      $aliases = $key['aliases'] ?? [];
      // Prepend alias with / if it does not have one
      if (strpos($values['alias'], '/') !== 0) {
        $values['alias'] = '/' . $values['alias'];
      }
      $aliases[$values['language']] = $values['alias'];

      $key['aliases'] = $aliases;
      $keys[$values['key']] = $key;

      $nodes[$values['key']] = $values['entity_id'];

      $this->config('frontend_routing.settings')
        ->set('keys', $keys)
        ->save();
    }

    $table = $form_state->getValue('table');
    if (is_array($table)) {
      foreach ($table as $key => $value) {
        $nodes[$key] = $value['entity_id'];
      }
      $this->config('frontend_routing.settings')
        ->set('keys', $keys)->save();
    }

    $this->updateNodeAliasSetting($nodes);
    parent::submitForm($form, $form_state);

  }

  /**
   * Update the pathauto setting for nodes in the frontend_routing.settings state.
   */
  private function updateNodeAliasSetting(array $nodes) {
    $this->state->set('frontend_routing.settings', $nodes);

    $nodes = $this->state->get('frontend_routing.settings') ?? [];
    foreach ($nodes as $node) {
      if ($node) {
        $node = $this->entityTypeManager->getStorage('node')->load($node);
        $paths = $node->get('path')->getValue();
        foreach ($paths as &$path) {
          $path['pathauto'] = FALSE;
        }
        $node->set('path', $paths);
        $node->save();
      }
    }
  }

}

==> frontend_routing-1.0.2.tar.gz: frontend_routing/src/Form/SettingsForm.php <==
<?php

declare(strict_types=1);

namespace Drupal\frontend_routing\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Language\LanguageInterface;
use Drupal\Core\State\StateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure Frontend routing settings for this site.
 */
final class SettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function __construct(
    protected ConfigFactoryInterface $config_factory,
    protected StateInterface $state,
    protected EntityTypeManagerInterface $entityTypeManager,
    protected EntityTypeBundleInfoInterface $entityTypeBundleInfo,
  ) {
    parent::__construct($config_factory);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('state'),
      $container->get('entity_type.manager'),
      $container->get('entity_type.bundle.info'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'frontend_routing_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['frontend_routing.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {

    $config = $this->config('frontend_routing.settings');
    $nodes = $this->state->get('frontend_routing.settings') ?? [];

    $keys = $config->get('keys') ?? [];

    $form['existing'] = [
      '#type' => 'details',
      '#title' => $this->t('Existing mappings'),
      '#open' => TRUE,
    ];

    $headers = [
      $this->t('Key'),
      $this->t('Node'),
      $this->t('Language'),
      $this->t('Alias'),
    ];

    $form['existing']['table'] = [
      '#type' => 'table',
      '#header' => $headers,
      '#tree' => TRUE,
      '#empty' => $this->t('No mappings found.'),
    ];

    foreach ($keys as $key => $value) {
      $aliases = $value['aliases'] ?? [];

      $form['existing']['table'][$key]['key'] = [
        '#title' => $this->t('Key'),
        '#type' => 'textfield',
        '#default_value' => $key,
        '#disabled' => TRUE,
      ];

      $form['existing']['table'][$key]['bundle'] = [
        '#title' => $this->t('Node Type'),
        '#type' => 'select',
        '#options' => $this->getNodeTypes(),
        '#required' => TRUE,
        '#default_value' => !empty($keys[$key]['bundle']) ? $keys[$key]['bundle'] : NULL,
      ];

      $required = TRUE;
      // If we have a generic alias, we do not require a node.
      if (!empty($aliases)) {
        $alias = reset($aliases);
        if (!empty($alias) && strpos($alias, '/:slug') !== FALSE) {
          $required = FALSE;
        }
      }

      $form['existing']['table'][$key]['entity_id'] = [
        '#title' => $this->t('Node'),
        '#type' => 'entity_autocomplete',
        '#target_type' => 'node',
        '#required' => $required,
        '#disabled' => !$required,
        '#default_value' => !empty($nodes[$key]) ? $this->entityTypeManager->getStorage('node')->load($nodes[$key]) : NULL,
      ];

      $form['existing']['table'][$key]['alias'] = [
        '#title' => $this->t('Aliases'),
        '#type' => 'item',
        // '#default_value' => $alias,
        '#markup' => implode(',', $aliases),
      ];

      $form['existing']['table'][$key]['delete'] = [
        '#value' => $this->t('Delete'),
        '#type' => 'submit',
        '#limit_validation_errors' => [],
        '#submit' => [
          [$this, 'deleteEntry'],
        ],
      ];

    }

    $form['details'] = [
      '#type' => 'details',
      '#title' => $this->t('Add new mapping'),
      '#open' => TRUE,
    ];

    $form['details']['key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Key'),
      '#description' => 'Unique key to identify the mapping.',
      '#maxlength' => 255,
    ];

    $form['details']['entity_id'] = [
      '#type' => 'entity_autocomplete',
      '#title' => $this->t('Node'),
      '#target_type' => 'node',
      '#maxlength' => 255,
    ];

    $form['details']['language'] = [
      '#type' => 'language_select',
      '#title' => $this->t('Language'),
      '#languages' => LanguageInterface::STATE_CONFIGURABLE,
    ];

    $form['details']['alias'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Alias'),
      '#description' => 'Starts with a /.',
      '#maxlength' => 255,
    ];
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {

    if ($form_state->getValue('key')) {
      if (empty($form_state->getValue('bundle'))) {
        $form_state->setErrorByName('bundle', $this->t('Node Type required.'));
      }
      if (empty($form_state->getValue('entity_id'))) {
        $form_state->setErrorByName('entity_id', $this->t('Node is required.'));
      }
      if (empty($form_state->getValue('language'))) {
        $form_state->setErrorByName('language', $this->t('Language is required.'));
      }
      if (empty($form_state->getValue('alias'))) {
        $form_state->setErrorByName('alias', $this->t('Alias is required.'));
      }
    }

    parent::validateForm($form, $form_state);
  }

  /**
   * Deletes an entry from the frontend_routing settings.
   *
   * @param array &$form
   *   The form array passed by reference.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object containing the user input and other data.
   *
   * @throws \Drupal\Core\Config\ConfigException
   *   In case of configuration errors.
   */
  public function deleteEntry(array &$form, FormStateInterface $form_state): void {
    $table = $form_state->getValue('table');
    $keys = $this->config('frontend_routing.settings')->get('keys') ?? [];

    // Remove the selected keys.
    foreach ($table as $key => $value) {
      unset($keys[$key]);
    }
    $this->config('frontend_routing.settings')
      ->set('keys', $keys)
      ->save();
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {

    $nodes = [];
    // Save data from the table.
    $keys = $this->config('frontend_routing.settings')->get('keys') ?? [];

    // Add a new item.
    $values = $form_state->getValues();
    if (!empty($values['key'])) {
      $key = !empty($keys[$values['key']]) ? $keys[$values['key']] : [];
      $aliases = $key['aliases'] ?? [];
      // Prepend alias with / if it does not have one.
      if (strpos($values['alias'], '/') !== 0) {
        $values['alias'] = '/' . $values['alias'];
      }
      $aliases[$values['language']] = $values['alias'];

      $key['aliases'] = $aliases;
      $keys[$values['key']] = $key;

      $nodes[$values['key']] = $values['entity_id'];

      $key['bundle'] = $values['bundle'];

      $this->config('frontend_routing.settings')
        ->set('keys', $keys)
        ->save();
    }

    $table = $form_state->getValue('table');
    if (is_array($table)) {
      foreach ($table as $key => $value) {
        $keys[$key]['bundle'] = $value['bundle'];
        if (!empty($value['entity_id'])) {
          $nodes[$key] = $value['entity_id'];
        }
      }
      $this->config('frontend_routing.settings')
        ->set('keys', $keys)->save();
    }

    $this->updateNodeAliasSetting($nodes);
    parent::submitForm($form, $form_state);

  }

  /**
   * Update the pathauto setting for nodes in the frontend_routing.settings state.
   */
  private function updateNodeAliasSetting(array $nodes) {
    $this->state->set('frontend_routing.settings', $nodes);
    $config = $this->config('frontend_routing.settings')->get('keys') ?? [];
    $nodes = $this->state->get('frontend_routing.settings') ?? [];

    // Update the alias for all related nodes.
    foreach ($nodes as $key => $node) {
      if ($node) {
        $node = $this->entityTypeManager->getStorage('node')->load($node);
        $aliases = !empty($config[$key]['aliases']) ? $config[$key]['aliases'] : [];

        // Iterate through all languages and check if we have a translation.
        foreach ($aliases as $language => $alias) {
          if ($node->hasTranslation($language)) {
            $node = $node->getTranslation($language);
          }
          else {
            continue;
          }
          $paths = $node->get('path')->getValue();
          foreach ($paths as &$path) {
            if ($path['langcode'] === $language) {
              // Save the alias and set pathauto to false.
              $alias = preg_replace('/^\/' . $language . '/', '', $alias);
              $path['alias'] = $alias;
              $path['pathauto'] = FALSE;
              $node->set('path', $paths);
              $node->save();
            }
          }
        }
      }
    }
  }

  /**
   * Get Node bundles.
   *
   * @return array
   */
  private function getNodeTypes() {
    $list = [];
    $bundles = $this->entityTypeBundleInfo->getBundleInfo('node');
    foreach ($bundles as $key => $bundle) {
      $list[$key] = $bundle['label'];
    }
    return $list;
  }

}
